package top.daxian1218.dust.commons.data;

import org.apache.commons.lang3.StringUtils;

import java.util.*;

/**
 * 列集合
 * @author hst
 */
public class HDataColumnCollection implements Iterable<HDataColumn>, Collection<HDataColumn> {

    private HDataTable table;
    private List<HDataColumn> data = new ArrayList<>();
    private Map<String, HDataColumn> idxData = new HashMap<>();

    HDataColumnCollection(HDataTable table)
    {
        this.table = table;
    }

    @Override
    public Iterator<HDataColumn> iterator() {
        return data.iterator();
    }

    @Override
    public Object[] toArray() {
        return data.toArray();
    }

    @Override
    public <T> T[] toArray(T[] a) {
        data.toArray(a);
        throw new UnsupportedOperationException();
    }

    public int size() {
        return data.size();
    }

    @Override
    public boolean isEmpty() {
        return data.isEmpty();
    }

    @Override
    public boolean contains(Object o) {
        return data.contains(o);
    }

    public boolean isReadOnly() {
        return !table.isEmpty();
    }

    public HDataColumn get(int index) {
        return  data.get(index);
    }

    public HDataColumn get(String name) {
        int i = getIndex(name);
        if (i < 0) {
            return null;
        }
        return  data.get(i);
    }

    public Integer getIndex(String name) {
        name = name.toUpperCase();
        if (!idxData.containsKey(name)) {
            return -1;
        }
        return  idxData.get(name).getIndex();
    }

    public boolean contains(String name) {
        if (StringUtils.isEmpty(name)) { return  false; }
        return idxData.containsKey(name.toUpperCase());
    }

    public boolean contains(HDataColumn column) {
        return data.contains(column);
    }

    public int indexOf(HDataColumn column) {
        return  data.indexOf(column);
    }

    public boolean add(HDataColumn col) {
        if (col == null) { throw new IllegalArgumentException("col"); }
        if (!data.contains(col) && !idxData.containsKey(col.getName().toUpperCase()))
        {
            data.add(col);
            col.setIndex(data.size() - 1);
            idxData.put(col.getName().toUpperCase(), col);
            return true;
        }

        return false;
    }

    private void resetColumnIndex() {
        for (int i = 0; i < data.size(); i++) {
            data.get(i).setIndex(i);
        }
    }

    private boolean remove(HDataColumn column, boolean resetIndex) {
        if (column == null)  { throw new NullPointerException("column"); }
        if (isReadOnly())  { throw new IllegalStateException("Table has data"); };

        idxData.remove(column.getName().toUpperCase());
        data.remove(column);
        if (resetIndex) {
            resetColumnIndex();
        }
        return true;
    }

    @Override
    public boolean remove(Object o) {
        HDataColumn column = (HDataColumn) o;
        return remove(column, true);
    }

    @Override
    public boolean containsAll(Collection<?> c) {
        return new HashSet<>(data).containsAll(c);
    }

    @Override
    public boolean addAll(Collection<? extends HDataColumn> c) {
        if (!table.isEmpty()) {
            throw new IllegalStateException("table has data");
        }

        for (HDataColumn dc : c) {
            add(dc);
        }

        return true;
    }

    @Override
    public boolean removeAll(Collection<?> c) {
        if (!table.isEmpty()) {
            throw new IllegalStateException("table has data");
        }

        for (Object o : c) {
            if (o instanceof HDataColumn) {
                remove((HDataColumn) o, false);
            }
        }
        resetColumnIndex();
        return true;
    }

    @Override
    public boolean retainAll(Collection<?> c) {
        if (!table.isEmpty()) {
            throw new IllegalStateException("table has data");
        }

        for (int i = data.size() - 1; i >= 0; i--) {
            HDataColumn col = data.get(i);
            if (!c.contains(col)) {
                remove(col, false);
            }
        }


        resetColumnIndex();
        return true;
    }

    public void clear() {
        if (isReadOnly()) { throw new IllegalStateException("table has data"); }
        data.clear();
        idxData.clear();
    }

    public void insert(int index, HDataColumn column) {
        if (column == null)  { throw new NullPointerException("column"); }
        if (isReadOnly()) { throw new IllegalStateException("table has data"); }

        if (data.contains(column)) {
            remove(column, false);
        }

        data.add(index, column);
        idxData.put(column.getName().toUpperCase(), column);
        resetColumnIndex();
    }

    public void removeAt(int index) {
        if (isReadOnly()) { throw new IllegalStateException("table has data");}

        if (index >= 0 && index < data.size()) {
            HDataColumn col = data.get(index);
            remove(col, false);

            for (int i = index; i < data.size(); i++) {
                data.get(i).setIndex(i);
            }
        }
    }

    void removeColumnAt(int index) {
        HDataColumn col = data.remove(index);
        idxData.remove(col.getName().toUpperCase());
        resetColumnIndex();
    }

    @Override
    protected void finalize() throws Throwable {
        super.finalize();
        data.clear();
        data = null;
        idxData.clear();
        idxData = null;
        table = null;
    }
}
