package top.daxian1218.dust.commons.data;

import com.alibaba.fastjson.JSONObject;
import top.daxian1218.dust.commons.generic.Preconditions;
import top.daxian1218.dust.commons.util.NameUtil;
import top.daxian1218.dust.commons.util.TypeUtil;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.stream.Stream;

/**
 * 一种简易的数据行映射，取代原来的Map方式存放数据
 *
 * @author hst
 */
public class HDataRecord implements IHDataRecord {
    private Map<String, Object> data = new LinkedHashMap<>();
    public final static long JS_NUMBER_MAX = (1L << 53) - 1;

    public HDataRecord() {

    }

    public  HDataRecord(Map<String, Object> map) {
        load(map);
    }

    @Override
    protected void finalize() throws Throwable {
        super.finalize();
        this.data.clear();
        this.data = null;
    }
    /**
     * 获取行内存放的数据长度
     *
     * @return 数据长度
     */
    @Override
    public int size() {
        return data.size();
    }

    @Override
    public boolean isEmpty() {
        return data.isEmpty();
    }

    @SuppressWarnings("MethodDoesntCallSuperMethod")
    @Override
    public IHDataRecord clone() {
        HDataRecord record = new HDataRecord();
        record.data.putAll(data);
        return record;
    }

    public boolean containsKey(String key) {
        Preconditions.checkNotEmpty(key);
        return data.containsKey(key.toUpperCase());
    }

    public boolean containsValue(Object value) {
        return data.containsValue(value);
    }

    @Override
    public IHDataRecord remove(String key) {
        String k = TypeUtil.toString(key);
        Preconditions.checkNotEmpty(k);
        data.remove(k.toUpperCase());
        return this;
    }

    @Override
    public IHDataRecord clear() {
        data.clear();
        return this;
    }


    /**
     * 获取指定列的值
     * 列名不区分大小写，空值无效
     *
     * @param column 列名
     * @return 值
     */
    public Object getValue(String column) {
        Preconditions.checkNotEmpty(column);
        column = column.toUpperCase();
        return data.get(column);
    }

    /**
     * 设置某一列的值，Object会自动转化为字符串存放
     * 列名为空值无效
     *
     * @param column
     * @param value
     * @return 返回DataRow对象，可完成连续操作
     */
    public HDataRecord setValue(String column, Object value) {
        Preconditions.checkNotEmpty(column);
        column = column.toUpperCase();
        data.put(column, value);
        return this;
    }


    public Object getValue(int index) {
        if (index < 0 || index >= data.size()) {
            return null;
        }
        return data.values().toArray()[index];
    }

    public HDataRecord setValue(int index, Object value) {
        if (index < 0 || index >= data.size()) {
            throw new IndexOutOfBoundsException();
        }

        String key = data.keySet().toArray(new String[0])[index];
        setValue(key, value);
        return this;
    }

    public IHDataRecord merge(IHDataRecord row) {
        row.forEachKey(k -> {
            data.put(k.toUpperCase(), row.getValue(k));
        });

        return this;
    }

    /**
     * 获取字符串值
     *
     * @param column
     * @return
     */
    public String getString(String column) {
        return TypeUtil.toString(getValue(column));
    }

    /**
     * 获取整数值
     *
     * @param column
     * @return
     */
    public Integer getInteger(String column) {
        return TypeUtil.toInteger(getValue(column));
    }

    /**
     * 获取长整数值
     *
     * @param column
     * @return
     */
    public Long getLong(String column) {
        return TypeUtil.toLong(getValue(column));
    }

    /**
     * 获取十进制值，无精度问题
     *
     * @param column
     * @return
     */
    public BigDecimal getDecimal(String column) {
        return TypeUtil.toDecimal(getValue(column));
    }

    /**
     * 获取布尔值
     *
     * @param column
     * @return
     */
    @Override
    public Boolean getBoolean(String column) {
        return TypeUtil.toBoolean(getValue(column));
    }

    /**
     * 获取日期时间
     *
     * @param column
     * @return
     */
    @Override
    public Date getDateTime(String column) {
        return TypeUtil.toDateTime(getValue((column)));
    }

    /**
     * 获取日期
     * @param column
     * @return
     */
    @Override
    public Date getDate(String column) {
        return TypeUtil.toDate(getValue((column)));
    }

    /**
     * 遍历行键值
     *
     * @param action
     */
    @Override
    public void forEachKey(Consumer<String> action) {
        data.forEach((k, v) -> {
            if (action != null) {
                action.accept(k);
            }
        });
    }

    @Override
    public IHDataRecord load(Map<String, Object> data) {
        Preconditions.checkNotNull(data);
        data.forEach((k, v) -> {
            this.data.put(k.toUpperCase(), v);
        });
        return this;
    }

    public void iterator(Function<String, Boolean> func) {
        for (String str : this.data.keySet()) {
            if (!func.apply(str)) {
                return;
            }
        }
    }

    private Object toJsonValue(Object v) {
        if (v instanceof Long) {
            long l = (long) v;
            return l > JS_NUMBER_MAX ? v.toString() : v;
        }

        if (v instanceof BigDecimal) {
            BigDecimal d = (BigDecimal) v;
            return d.longValue() > JS_NUMBER_MAX ? v.toString() : v;
        }
        return v;
    }

    public JSONObject toJSON() {
        return toJSON(HDataConfig.Instance.isCamelName());
    }

    public JSONObject toJSON(boolean camelName) {
        JSONObject json = new JSONObject();
        if (camelName) {
            this.data.forEach((k, v) -> {
                json.put(NameUtil.underscore2Camel(k), toJsonValue(v));
            });
            return json;
        }

        this.data.forEach((k, v) -> {
            json.put(k, toJsonValue(v));
        });
        return json;
    }

    @Override
    public Map<String, Object> toMap() {
        Map<String, Object> map = new HashMap<>(this.data.size());
        map.putAll(this.data);
        return map;
    }

    @Override
    public Iterator<String> iterator() {
        return data.keySet().iterator();
    }

    @Override
    public Stream<String> stream() {
        return this.data.keySet().stream();
    }

    @Override
    public Collection<Object> values() {
        return data.values();
    }
}
