package top.daxian1218.dust.commons.data;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.ObjectUtils;
import top.daxian1218.dust.commons.exception.DustSystemException;
import top.daxian1218.dust.commons.generic.Preconditions;
import top.daxian1218.dust.commons.util.ListUtil;
import top.daxian1218.dust.commons.util.NameUtil;
import top.daxian1218.dust.commons.util.TypeUtil;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 一种简易的数据行映射，取代原来的Map方式存放数据
 *
 * @author hst
 */
public class HDataRow implements Iterable<Object> {
    private List<Object> data = new ArrayList<>();
    private List<Object> originData;

    private boolean detached = true;

    private HDataTable table;
    public final static long JS_NUMBER_MAX = (1L << 53) - 1;

    HDataRow(HDataTable table) {
        this.table = table;
    }

    @Override
    protected void finalize() throws Throwable {
        super.finalize();
        this.data.clear();
        this.data = null;
        this.table = null;
        if (this.originData != null) {
            this.originData.clear();
        }
        this.originData = null;
    }

    public HDataTable getTable() {
        return table;
    }

    /**
     * 获取行内存放的数据长度
     *
     * @return 数据长度
     */
    public int size() {
        return data.size();
    }

    public boolean isEmpty() {
        return data.isEmpty();
    }

    public boolean containsColumn(String k) {
        Preconditions.checkNotEmpty(k);
        return table.getColumns().contains(k);
    }

    public boolean containsValue(Object value) {
        return data.contains(value);
    }

    public HDataRow put(String key, Object value) {
        return setValue(key, value);
    }

    public HDataRow putAll(Map<? extends String, ?> m) {
        m.forEach(this::setValue);
        return this;
    }

    public void clear() {
        data.clear();
    }

    public Set<String> keySet() {
        return table.getColumns().stream().map(HDataColumn::getName).collect(Collectors.toSet());
    }

    public Collection<Object> values() {
        return data;
    }

    /**
     * 获取指定列的值
     * 列名不区分大小写，空值无效
     *
     * @param column
     * @return 值
     * @deprecated  改为getValue
     * @see HDataRow#getValue(String)
     */
    @Deprecated
    public Object get(String column) {
        return getValue(column);
    }

    public Object getValue(String column) {
        return getValue(column, false);
    }

    public Object getValue(String column, boolean original) {
        Preconditions.checkNotEmpty(column);
        column = column.toUpperCase();
        HDataColumn col = table.getColumns().get(column);
        if (col == null) {
            return null;
        }
        return getValue(col, original);
    }

    public Object getValue(HDataColumn col, boolean original) {
        if (col == null) {
            return null;
        }

        return getValue(col.getIndex(), original);
    }

    public Object getValue(int index) {
        return getValue(index, false);
    }

    public Object getValue(int index, boolean original) {
        if (index < 0 || index >= data.size()) {
            return null;
        }

        if (original && originData != null) {
            return ListUtil.getValueOrNull(originData, index);
        }

        return ListUtil.getValueOrNull(data, index);
    }


    /**
     * 设置某一列的值，Object会自动转化为字符串存放
     * 列名为空值无效
     *
     * @param column
     * @param value
     * @return 返回DataRow对象，可完成连续操作
     */
    @Deprecated
    public HDataRow set(String column, Object value) {
        return setValue(column, value);
    }

    public HDataRow setValue(String column, Object value) {
        Preconditions.checkNotEmpty(column);
        column = column.toUpperCase();
        HDataColumn col = table.getColumns().get(column);
        if (col == null) {
            col = table.addColumn(column);
        }

        ListUtil.setValueOrNull(data, col.getIndex(), value);
        return this;
    }

    public void setValue(int index, Object value) {
        if (index < 0 || index >= table.getColumns().size()) {
            throw new IndexOutOfBoundsException("index is out of column range");
        }

        String name = table.getColumns().get(index).getName();
        setValue(name, value);
    }

    public void merge(HDataRow row) {
        table.getColumns().forEach(c -> {
            Object v = row.get(c.getName());
            if (v != null) {
                data.set(c.getIndex(), v);
            }
        });
    }

    public void append(HDataRow row) {
        row.forEachColumn(c -> {
            setValue(c, row.getValue(c));
        });
    }

    /**
     * 获取字符串值
     *
     * @param column
     * @return
     */
    public String getString(String column) {
        return TypeUtil.toString(get(column));
    }

    /**
     * 获取整数值
     *
     * @param column
     * @return
     */
    public int getInteger(String column) {
        return TypeUtil.toInteger(get(column));
    }

    /**
     * 获取长整数值
     *
     * @param column
     * @return
     */
    public long getLong(String column) {
        return TypeUtil.toLong(get(column));
    }

    /**
     * 获取十进制值，无精度问题
     *
     * @param column
     * @return
     */
    public BigDecimal getDecimal(String column) {
        return TypeUtil.toDecimal(get(column));
    }

    /**
     * 获取布尔值
     *
     * @param column
     * @return
     */
    public boolean getBoolean(String column) {
        return TypeUtil.toBoolean(get(column));
    }

    /**
     * 获取日期时间
     *
     * @param column
     * @return
     */
    public Date getDateTime(String column) {
        return TypeUtil.toDateTime(get((column)));
    }

    /**
     * 遍历行键值
     *
     * @param action
     */
    public void forEachColumn(Consumer<String> action) {
        table.getColumns().forEach((c) -> {
            if (action != null) {
                action.accept(c.getName());
            }
        });
    }

    public void forEach(BiConsumer<String, Object> action) {
        table.getColumns().forEach((c) -> {
            if (action != null) {
                action.accept(c.getName(), getValue(c.getIndex()));
            }
        });
    }

    public void load(Map<String, Object> data) {
        Preconditions.checkNotNull(data);
        table.getColumns().forEach(c -> {
            if (data.containsKey(c.getName())) {
                set(c.getName(), data.get(c.getName()));
            }
        });
    }

    public void iterator(Function<String, Boolean> func) {
        for (HDataColumn column : this.table.getColumns()) {
            if (!func.apply(column.getName())) {
                return;
            }
        }
    }

    private Object toJsonValue(Object v) {
        if (v instanceof Long) {
            long l = (long) v;
            return l > JS_NUMBER_MAX ? v.toString() : v;
        }

        if (v instanceof BigDecimal) {
            BigDecimal d = (BigDecimal) v;
            return d.longValue() > JS_NUMBER_MAX ? v.toString() : v;
        }

        return v;
    }

    public JSONObject toJSON() {
        if (HDataConfig.Instance.isCamelName()) {
            return toJSON(true);
        }

        return toJSON(false);
    }

    public JSONArray toJSONArray() {
        JSONArray arr = new JSONArray();
        this.table.getColumns().forEach(col -> {
            arr.add(toJsonValue(getValue(col, false)));
        });

        return arr;
    }

    public JSONObject toJSON(boolean camelName) {
        JSONObject json = new JSONObject();
        if (camelName) {
            this.table.getColumns().forEach((column) -> {
                json.put(NameUtil.underscore2Camel(column.getName()), toJsonValue(getValue(column, false)));
            });
        }

        this.table.getColumns().forEach((column) -> {
            json.put(column.getName(), toJsonValue(getValue(column, false)));
        });
        return json;
    }

    public Map<String, Object> toMap() {
        HashMap<String, Object> map = new HashMap<>(this.table.getColumns().size());
        this.table.getColumns().forEach(c -> {
            map.put(c.getName(), getValue(c.getIndex()));
        });
        return map;
    }


    @Override
    public Iterator<Object> iterator() {
        if (table.getColumns().size() <= 0) {
            return data.iterator();
        }

        return new Iterator<Object>() {
            private int index = 0;

            @Override
            public boolean hasNext() {
                return table.getColumns().size() > 0 && index < table.getColumns().size();
            }

            @Override
            public Object next() {
                return getValue(index++);
            }
        };
    }

    public HDataRow moveTo(int index) {
        if (this.table.isEmpty()) {
            return this;
        }

        if (index < 0) {
            index = this.table.size() + index;
            if (index < 0) {
                index = 0;
            }
        }

        if (index >= this.table.size()) {
            index = this.table.size() - 1;
        }

        if (index == this.table.indexOf(this)) {
            return this;
        }

        this.table.getRows().moveRow(this, index);
        return this;
    }

    public boolean isDetached() {
        return detached;
    }

    void setDetached(boolean detached) {
        this.detached = detached;
    }

    public void acceptChanges() {
        this.originData = new ArrayList<>(this.data);
    }

    public void rejectChanges() {
        if (this.originData == null) {
            this.data = new ArrayList<>();
            return;
        }

        this.data = new ArrayList<>(this.originData);
    }

    public boolean isChanged(String column) {
        HDataColumn col = table.getColumns().get(column);
        if (col == null) {
            return false;
        }

        if (this.originData == null) {
            return true;
        }

        return ObjectUtils.notEqual(this.originData.get(col.getIndex()), getValue(col.getIndex()));
    }

    void removeAt(int index) {
        if (data.size() > index) {
            data.remove(index);
        }

        if (this.originData != null && this.originData.size() > index) {
            originData.remove(index);
        }
    }

    @Override
    public String toString() {
        return data.toString();
    }
}
