package top.daxian1218.dust.commons.data;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import top.daxian1218.dust.commons.generic.Preconditions;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 一种简易的表映射类，方便存取数据，取代原有List Map的方式
 * 不区分大小写
 *
 * @author hst
 */
public class HDataTable implements Iterable<HDataRow> {
    private HDataRowCollection rows = new HDataRowCollection(this);
    private HDataColumnCollection columns = new HDataColumnCollection(this);

    public HDataTable() {
    }

    @Override
    protected void finalize() throws Throwable {
        super.finalize();
        this.rows.finalize();
        this.rows = null;
        this.columns.finalize();
        this.columns = null;
    }

    /**
     * 根据索引获取行
     * @param index
     * @return
     */
    public HDataRow get(Integer index) {
        return rows.get(index);
    }

    /**
     * 获取列集合
     * @return 列集合
     */
    public HDataColumnCollection getColumns() {
        return columns;
    }

    /**
     * 获取所有行
     * @return 行集合
     */
    public HDataRowCollection getRows() {
        return rows;
    }

    /**
     * 获取第一行第一列的值
     * @return 值，找不到则为null
     */
    public Object getScalar() {
        return rows.size() > 0 ? rows.get(0).getValue(0) : null;
    }

    /**
     * 新增行数据     *
     * @return 新加入的行
     */
    public HDataRow addRow() {
        HDataRow row = new HDataRow(this);
        rows.add(row);
        return row;
    }

    /**
     * 加入行到指定索引
     * @param i 位置, 不允许超过size
     * @return 新加入的行
     */
    public HDataRow insertRow(int i) {
        HDataRow row = new HDataRow(this);
        rows.add(i, row);
        return row;
    }

    public HDataRow removeRow(HDataRow row) {
        rows.remove(row);
        return row;
    }

    public HDataRow removeRowAt(int index) {
        return rows.removeAt(index);
    }

    public  HDataColumn addColumn(String name) {
        columns.add(new HDataColumn(name));
        return columns.get(columns.size() - 1);
    }

    public int size() {
        return this.rows.size();
    }

    public boolean isEmpty() {
        return rows.isEmpty();
    }

    public boolean contains(HDataRow o) {
        return rows.contains(o);
    }

    public JSONObject toGridJSON() {
        JSONArray colArr = new JSONArray(columns.size());
        this.columns.forEach(col -> {
            colArr.add(col.getName());
        });

        JSONArray rowArr = new JSONArray(rows.size());
        for (HDataRow r : rows) {
            rowArr.add(r.toJSONArray());
        }

        JSONObject grid = new JSONObject();
        grid.put("columns", colArr);
        grid.put("rows", rowArr);
        return grid;
    }

    public JSONArray toDataJSON() {
        JSONArray arr = new JSONArray();
        for (HDataRow r : rows) {
            arr.add(r.toJSON());
        }
        return arr;
    }

    public JSONArray toDataJSON(boolean camelName) {
        JSONArray arr = new JSONArray();
        for (HDataRow r : rows) {
            arr.add(r.toJSON(camelName));
        }
        return arr;
    }


    @Override
    public String toString() {

        return "{" + "trowCount:" + rows.size() + "}";
    }

    /**
     * 遍历数据，iteration返回值为是否继续，true继续，false结束遍历
     *
     * @param iterator 迭代函数
     */
    public void iterate(Function<HDataRow, Boolean> iterator) {
        Preconditions.checkNotNull(iterator);
        for (HDataRow r : rows) {
            if (!iterator.apply(r)) {
                return;
            }
        }
    }

    @Override
    public Iterator<HDataRow> iterator() {
        return rows.iterator();
    }

    public List<HDataRow> toList() {
        return rows.stream().collect(Collectors.toList());
    }

    public void clear() {
        rows.clear();
    }

    public HDataRow get(int index) {
        return rows.get(index);
    }

    public int indexOf(HDataRow o) {
        return rows.indexOf(o);
    }


    public int lastIndexOf(HDataRow o) {
        return rows.lastIndexOf(o);
    }

    public List<HDataRow> subList(int fromIndex, int toIndex) {
        return rows.subList(fromIndex, toIndex);
    }

    public Stream<HDataRow> stream() {
        return rows.stream();
    }

    public  void acceptChanges() {
        this.rows.stream().forEach(HDataRow::acceptChanges);
    }

    public static HDataTable load(InputStream stream) throws IOException {
        HDataTable table = new HDataTable();
        BufferedReader br = new BufferedReader(new InputStreamReader(stream));
        String str = br.readLine();
        String[] columns = str.split(",");
        for (String column : columns) {
            table.addColumn(column);
        }

        while ((str = br.readLine()) != null) {
            String[] values = str.split(",");

            int len = Math.min(values.length, table.getColumns().size());
            HDataRow newRow = table.addRow();
            for (int i = 0; i < len; i++) {
                newRow.setValue(i, values[i]);
            }
        }

        stream.close();
        return table;
    }

    public static HDataTable load(String csvPath) throws IOException {
        File file = new File(csvPath);
        if (!file.exists() || !file.isFile()) {
            return new HDataTable();
        }
        return load(Files.newInputStream(Paths.get(csvPath)));
    }

    public void removeColumnAt(Integer index) {
        this.rows.removeColumnAt(index);
        this.columns.removeColumnAt(index);
    }
}
